@@ -1,8 +1,11 @@
 """Provide a model for the Z-Wave JS controller."""
-from dataclasses import asdict, dataclass
+from dataclasses import dataclass
 from typing import TYPE_CHECKING, Any, Dict, List, Literal, Optional, Union, cast
 
-from zwave_js_server.model.firmware import FirmwareUpdateFileInfo, FirmwareUpdateInfo
+from zwave_js_server.model.node.firmware import (
+    NodeFirmwareUpdateFileInfo,
+    NodeFirmwareUpdateInfo,
+)
 
 from ...const import (
     MINIMUM_QR_STRING_LENGTH,
@@ -20,6 +23,7 @@ from ..association import AssociationAddress, AssociationGroup
 from ..node import Node
 from .data_model import ControllerDataType
 from .event_model import CONTROLLER_EVENT_MODEL_MAP
+from .firmware import ControllerFirmwareUpdateProgress, ControllerFirmwareUpdateResult
 from .inclusion_and_provisioning import (
     InclusionGrant,
     ProvisioningEntry,
@@ -88,6 +92,13 @@ class Controller(EventBase):
         """Return own_node_id."""
         return self.data.get("ownNodeId")
 
+    @property
+    def own_node(self) -> Optional[Node]:
+        """Return own_node."""
+        if self.own_node_id is None:
+            return None
+        return self.nodes.get(self.own_node_id)
+
     @property
     def is_primary(self) -> Optional[bool]:
         """Return is_primary."""
@@ -175,6 +186,13 @@ class Controller(EventBase):
         """Return inclusion state."""
         return InclusionState(self.data["inclusionState"])
 
+    @property
+    def rf_region(self) -> Optional[RFRegion]:
+        """Return RF region of controller."""
+        if (rf_region := self.data.get("rfRegion")) is None:
+            return None
+        return RFRegion(rf_region)
+
     def update(self, data: ControllerDataType) -> None:
         """Update controller data."""
         self.data = data
@@ -713,32 +731,34 @@ class Controller(EventBase):
         return cast(bool, data["progress"])
 
     async def async_get_available_firmware_updates(
-        self, node: Node, api_key: str
-    ) -> List[FirmwareUpdateInfo]:
+        self, node: Node, api_key: str, include_prereleases: bool = False
+    ) -> List[NodeFirmwareUpdateInfo]:
         """Send getAvailableFirmwareUpdates command to Controller."""
         data = await self.client.async_send_command(
             {
                 "command": "controller.get_available_firmware_updates",
                 "nodeId": node.node_id,
                 "apiKey": api_key,
+                "includePrereleases": include_prereleases,
             },
-            require_schema=23,
+            require_schema=24,
         )
         assert data
-        return [FirmwareUpdateInfo.from_dict(update) for update in data["updates"]]
+        return [NodeFirmwareUpdateInfo.from_dict(update) for update in data["updates"]]
 
-    async def async_begin_ota_firmware_update(
-        self, node: Node, update: FirmwareUpdateFileInfo
-    ) -> None:
-        """Send beginOTAFirmwareUpdate command to Controller."""
-        await self.client.async_send_command(
+    async def async_firmware_update_ota(
+        self, node: Node, updates: List[NodeFirmwareUpdateFileInfo]
+    ) -> bool:
+        """Send firmwareUpdateOTA command to Controller."""
+        data = await self.client.async_send_command(
             {
-                "command": "controller.begin_ota_firmware_update",
+                "command": "controller.firmware_update_ota",
                 "nodeId": node.node_id,
-                "update": asdict(update),
+                "updates": [update.to_dict() for update in updates],
             },
-            require_schema=21,
+            require_schema=24,
         )
+        return cast(bool, data["success"])
 
     def receive_event(self, event: Event) -> None:
         """Receive an event."""
@@ -764,6 +784,18 @@ class Controller(EventBase):
         event.data["controller"] = self
         self.emit(event.type, event.data)
 
+    def handle_firmware_update_progress(self, event: Event) -> None:
+        """Process a firmware update progress event."""
+        event.data["firmware_update_progress"] = ControllerFirmwareUpdateProgress(
+            event.data["progress"]
+        )
+
+    def handle_firmware_update_finished(self, event: Event) -> None:
+        """Process a firmware update finished event."""
+        event.data["firmware_update_finished"] = ControllerFirmwareUpdateResult(
+            event.data["result"]
+        )
+
     def handle_inclusion_failed(self, event: Event) -> None:
         """Process an inclusion failed event."""
 
